/*
 * Copyright (c) 2012, Masanori Miyamoto.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * 1.Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 2.Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 */ 
#ifndef __CZARBASIC_VM_H__
#define __CZARBASIC_VM_H__

#define VMMEM_MAX	65536	/* プログラムメモリサイズ */
/* プログラムメモリ */
extern int g_vmmem[VMMEM_MAX];
extern int g_vmmem_last;
/* プログラムカウンタ */
extern int g_pc;

#define STRING_POOL_MAX	65536	/* 文字列プールサイズ */
/* 文字列プール */
extern char g_string_pool[STRING_POOL_MAX];

/* 行番号テーブル構造体 */
struct line_num_table {
	int line_num;	/* 行番号 */
	int address;	/* アドレス */
};
#define LINE_NUM_MAX	65536	/* 行番号テーブル数 */
/* 行番号テーブル */
extern struct line_num_table g_line_num_table[LINE_NUM_MAX];

/* vm実行 */
void vm_run();

/* ディスアセンブル */
void vm_disasm();

/* オペコード一覧 */
enum opecode {
	OP_NOP,
	OP_PUSH_INT,		/* 数値をスタックに積む */
	OP_PUSH_VAR,		/* 変数をスタックに積む */
	OP_POP_VAR,		/* 変数にスタックから値を代入 */
	OP_ADD,			/* 加算 */
	OP_SUB,			/* 減算 */
	OP_MUL,			/* 乗算 */
	OP_DIV,			/* 除算 */
	OP_CMP_EQ,		/* 比較(=) */
	OP_CMP_NE,		/* 比較(<>) */
	OP_CMP_GT,		/* 比較(>) */
	OP_CMP_GE,		/* 比較(>=) */
	OP_CMP_LT,		/* 比較(<) */
	OP_CMP_LE,		/* 比較(<=) */
	OP_COND_JMP,		/* 条件ジャンプ */
	OP_JMP,			/* 無条件ジャンプ */
	OP_JMP_NUM,		/* 行番号ジャンプ */
	OP_CALL,		/* 行番号サブルーチンコール */
	OP_RET,			/* サブルーチンコールリターン */
	OP_INPUT,		/* INPUTステートメント */
	OP_PRN_STRING,		/* 文字列出力 */
	OP_PRN,			/* スタック上の値を出力 */
	OP_CLEAR,		/* 変数のクリア */
	OP_END,			/* 終了 */
	OP_NCOND_JMP,		/* 条件ジャンプ(否定)*/
	OP_LD_EL,		/* エディタでの行番号を設定 */
	OP_LD_BL,		/* BASICでの行番号を設定 */
};

#endif
