/*
 * Copyright (c) 2012, Masanori Miyamoto.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * 1.Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 2.Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 */ 
#include <stdio.h>
#include <stdlib.h>
#include "CzarBasic.h"
#include "CzarBasic_vm.h"
#include "CzarBasic_error.h"

static const char *st_err_message[] = {
	"Stack overflow",
	"Stack underflow",
	"Out of range",
	"Undefined line number",
	"Buffer overflow",
	"Program memory overflow",
	"Out of string space",
	"Line number table overflow",
	"Line number error",
	"Syntax error",
	"Overflow error",
	"Out of memory",
	"Underflow error",
	"Division by zero",
	"Already defined line number",
};

/* エラー終了 */
void ErrorExit(ERROR_CODE errno, int line_num, int basic_line_num)
{
#ifdef PRINT_DISASM
	/* ディスアセンブル */
	vm_disasm();
#endif
	/* エラーメッセージ表示 */
	fprintf(stderr, "%s", st_err_message[errno]);
	if(line_num >= 0) {
		fprintf(stderr, " in %d", line_num);
	}
	if(basic_line_num != -1) {
		fprintf(stderr, ":%d", basic_line_num);
	}
	fprintf(stderr, "\n");
	exit(1);
}

